"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.denormalizeDocument = denormalizeDocument;

var _HTypes = require("./HTypes");

var _HVisit = require("./HVisit");

var _HDocument = require("./HDocument");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const elementUid = (elementType, elementId) => `${elementType}:${elementId}`;

function denormalizeDocument(doc) {
  const nodes = new Map(); // I will need two visits. One to create all the nodes while setting the parents, and
  // a second one where we change the children.

  (0, _HVisit.visitDocument)(doc, (normalizedDoc, nodeType, nodeId) => {
    const element = (0, _HDocument.mappedElement)(doc.maps, nodeType, nodeId);
    const parentUid = element.parentType && element.parentId ? elementUid(element.parentType, element.parentId) : null;
    const parent = parentUid ? nodes.get(parentUid) || null : null;

    const denormalizedNode = _objectSpread(_objectSpread({}, element), {}, {
      parent
    });

    nodes.set(elementUid(nodeType, nodeId), denormalizedNode);
  });
  (0, _HVisit.visitDocument)(doc, (normDoc, nodeType, nodeId) => {
    const element = (0, _HDocument.mappedElement)(doc.maps, nodeType, nodeId);
    const uid = elementUid(nodeType, nodeId);
    const denormalizedNode = nodes.get(uid);
    const typeLinks = doc.schema.types[nodeType];

    for (const fieldName in typeLinks) {
      if (fieldName === 'parentId') {
        continue;
      }

      const linkProps = typeLinks[fieldName];

      if (Array.isArray(linkProps)) {
        const linkedIds = element[fieldName];

        if (!Array.isArray(linkedIds)) {
          denormalizedNode[fieldName] = [];
        } else {
          denormalizedNode[fieldName] = linkedIds.map(childId => nodes.get(elementUid(linkProps[0].__schemaType, childId))).filter(childNode => childNode !== undefined);
        }
      } else {
        denormalizedNode[fieldName] = null;
      }
    }
  }, {}, _HTypes.DocumentVisitTraversal.DEPTH_FIRST);
  return nodes.get(elementUid(doc.rootType, doc.rootId));
}
//# sourceMappingURL=HDenormalize.js.map