import { HDocOperation, Id, IDocumentSchema, INormalizedDocument, INormalizedMutableMapsDocument, IParentedId, MappedParentedTypesFields, MapsOfNormDoc, UOfNormDoc } from './HTypes';
/**
 * Returns a list of HDocOperations that if applied
 * will transform baseDoc in laterDoc. laterDoc
 * is assumed to have the same root element and schema as
 * baseDoc
 *
 * @param {INormalizedDocument<MapsInterface, U>} baseDoc
 * @param {INormalizedDocument<MapsInterface, U>} laterDoc
 * @returns {HDocOperation<MapsInterface, any, U>[]}
 */
export declare function diff<NorDoc extends INormalizedDocument<any, any>>(baseDoc: NorDoc, laterDoc: NorDoc): HDocOperation<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>, any>[];
/**
 * Given a base version and a later version of a document element
 * with schema schema and type elementType, returns a Partial version
 * of the element containing the info fields that have changed between baseEl
 * and laterEl.
 *
 * @param {IDocumentSchema<MapsInterface, U>} schema
 * @param {U} elementType
 * @param {T} baseEl
 * @param {T} laterEl
 * @returns {Partial<T>}
 */
export declare function diffElementInfo<MapsInterface, U extends keyof MapsInterface, T extends IParentedId<U, U>>(schema: IDocumentSchema<MapsInterface, U>, elementType: U, baseEl: T, laterEl: T): Partial<T>;
export declare function diffInfoOf<MapsInterface, U extends keyof MapsInterface, T extends MappedParentedTypesFields<MapsInterface>>(baseDoc: INormalizedDocument<MapsInterface, U> | INormalizedMutableMapsDocument<MapsInterface, U>, laterDoc: INormalizedDocument<MapsInterface, U> | INormalizedMutableMapsDocument<MapsInterface, U>, elementType: U, elementId: Id): Partial<T>;
