import { AllMappedTypesFields, EntitiesMaps, HDocOperation, Id, IInsertElement, ILazyMutableMap, IMoveElement, IMutableDocument, INormalizedDocument, INormalizedMutableMapsDocument, IParentedId, MapsOfNormDoc, MutableEntitiesMaps, Path, UOfNormDoc } from './HTypes';
export declare function isId(obj: any): obj is Id;
export declare function isNullableId(obj: any): obj is Id | null;
export declare function isParentedId(obj: any): obj is IParentedId;
export declare function isParentedMap(obj: any): obj is Map<Id, IParentedId>;
/**
 * Creates a shallow clone of the document. The maps are new objects,
 * but the entities mapped are the same as the original.
 *
 * The rationale for the shallow version of the elements is that
 * changes will be performed as setting new versions in the dictionary, rather
 * than direct manipulation of the objects.
 *
 * @param {INormalizedDocument<MapsInterface, U>} doc The document to be cloned
 * @returns {INormalizedDocument<MapsInterface, U>} shallow clone of the document
 */
export declare function cloneNormalizedDocument<NorDoc extends INormalizedDocument<any, any>>(doc: NorDoc): NorDoc;
/**
 * Creates an empty copy version of the normalized document
 * passed as parameter.
 *
 * @returns {INormalizedDocument<MapsInterface, U>}
 */
export declare function clearedNormalizedDocument<NorDoc extends INormalizedDocument<any, any>>(doc: NorDoc): NorDoc;
export declare function idAndTypeForPath<MapsInterface, U extends keyof MapsInterface>(doc: INormalizedDocument<MapsInterface, U> | INormalizedMutableMapsDocument<MapsInterface, U>, path: Path<MapsInterface>): {
    __typename: U;
    _id: Id;
};
export declare function parentToChildFieldName<MapsInterface, U extends keyof MapsInterface>(document: INormalizedDocument<MapsInterface, U> | INormalizedMutableMapsDocument<MapsInterface, U>, parentType: U, childType: U): AllMappedTypesFields<MapsInterface>;
/**
 * Given an element type and id of and element within the document,
 * returns the path to it.
 *
 * Each element is expected to be referenced only once as an official child
 * within the document.
 *
 * The element could still be referenced informally from fields not captured
 * in the document schema
 *
 * @param {U} elementTypeMap
 * @param {Id} elementId
 * @returns {Path}
 */
export declare function pathForElementWithId<NorDoc extends INormalizedDocument<any, any>>(doc: INormalizedDocument<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>> | INormalizedMutableMapsDocument<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>>, elementTypeMap: UOfNormDoc<NorDoc>, elementId: Id): Path<MapsOfNormDoc<NorDoc>>;
export declare function mutableDocument<NorDoc extends INormalizedDocument<any, any>>(doc: NorDoc): IMutableDocument<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>, NorDoc>;
/**
 * Checks if the object passed as parameter is a mutable
 * lazy map with Ids as keys and IParentedId subtypes as
 * values.
 *
 * @param obj
 * @returns {obj is ILazyMutableMap<Id, IParentedId>}
 */
export declare function isParentedMutableMap(obj: any): obj is ILazyMutableMap<Id, IParentedId>;
/**
 * Checks if an element of type elementType with id elementId is in the document
 * hierarchy.
 *
 * @param {EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>} maps
 * @param {U} elementType
 * @param {Id} elementId
 * @returns {boolean}
 */
export declare function hasMappedElement<MapsInterface, U extends keyof EntitiesMaps<MapsInterface> = keyof EntitiesMaps<MapsInterface>>(maps: EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>, elementType: U, elementId: Id): boolean;
/**
 * Given a map of entities or mutable entities, returns the element that corresponds to the type
 * and id provided as parameter.
 *
 * If the type requested is not mapped, a type error is thrown. If no element with the requested _id is found,
 * a reference error is thrown.
 *
 * @param {EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>} maps
 * @param {U} elementType
 * @param {Id} elementId
 * @returns {EntitiesMaps<MapsInterface>[U] extends Map<Id, infer T> ? T : never}
 */
export declare function mappedElement<MapsInterface, U extends keyof EntitiesMaps<MapsInterface> = keyof EntitiesMaps<MapsInterface>>(maps: EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>, elementType: U, elementId: Id): EntitiesMaps<MapsInterface>[U] extends Map<Id, infer T> ? T : never;
export declare const docReducer: <NorDoc extends INormalizedDocument<any, any>>(doc: NorDoc, cmd: IInsertElement<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>, any> | import("./HTypes").IChangeElement<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>, any> | import("./HTypes").IDeleteElement<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>> | IMoveElement<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>, any> | HDocOperation<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>, any>[]) => NorDoc;
