"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LazyMutableMap = void 0;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class LazyMutableMap {
  constructor(originalMap) {
    _defineProperty(this, "originalValuesMap", void 0);

    _defineProperty(this, "valuesMap", void 0);

    _defineProperty(this, "added", void 0);

    _defineProperty(this, "changed", void 0);

    _defineProperty(this, "deleted", void 0);

    _defineProperty(this, "hasChanged", () => {
      return this.originalValuesMap !== this.valuesMap;
    });

    _defineProperty(this, "set", (key, value) => {
      if (this.valuesMap === this.originalValuesMap && this.valuesMap.has(key) && this.originalValuesMap.get(key) === value) {
        return this;
      }

      this.createCopyIfNeeded();
      this.valuesMap.set(key, value);

      if (this.deleted.has(key)) {
        this.deleted.delete(key);
      }

      if (this.originalValuesMap.has(key)) {
        if (this.originalValuesMap.get(key) !== value) {
          this.changed.set(key, value);
        } else {
          if (this.changed.has(key)) {
            this.changed.delete(key);
          }
        }
      } else {
        this.added.set(key, value);
      }

      return this;
    });

    _defineProperty(this, "get", key => this.valuesMap.get(key));

    _defineProperty(this, "has", key => this.valuesMap.has(key));

    _defineProperty(this, "delete", key => {
      if (!this.valuesMap.has(key)) {
        return false;
      }

      this.createCopyIfNeeded();
      this.deleted.add(key);

      if (this.added.has(key)) {
        this.added.delete(key);
      }

      if (this.changed.has(key)) {
        this.changed.delete(key);
      }

      return this.valuesMap.delete(key);
    });

    _defineProperty(this, "getMap", () => this.valuesMap);

    _defineProperty(this, "keys", () => this.valuesMap.keys());

    _defineProperty(this, "values", () => this.valuesMap.values());

    _defineProperty(this, "createCopyIfNeeded", () => {
      if (this.originalValuesMap === this.valuesMap) {
        this.valuesMap = new Map(this.originalValuesMap.entries());
      }
    });

    _defineProperty(this, "getOriginal", key => this.originalValuesMap.get(key));

    _defineProperty(this, "delta", () => {
      return {
        added: this.added,
        changed: this.changed,
        deleted: this.deleted
      };
    });

    this.valuesMap = this.originalValuesMap = originalMap;
    this.added = new Map();
    this.changed = new Map();
    this.deleted = new Set();
  }

}

exports.LazyMutableMap = LazyMutableMap;
//# sourceMappingURL=LazyMap.js.map